﻿using System;
using System.Diagnostics;
using System.Globalization;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Forms;
using System.Windows.Threading;
using TSLab.DataSource;
using Button = System.Windows.Controls.Button;
using MessageBox = System.Windows.MessageBox;

namespace GuiBinConverter
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private readonly ObservableCollection<TickerInfo> _tickers = new ObservableCollection<TickerInfo>();

        public MainWindow()
        {
            InitializeComponent();

            cbxTickers.ItemsSource = _tickers;
        }

        private void btnBrowseBin_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new FolderBrowserDialog();
            if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                tbxBinPath.Text = dialog.SelectedPath;
        }

        private void btnLoadBinTickers_Click(object sender, RoutedEventArgs e)
        {
            var tickerInfos = new List<TickerInfo>();
            _tickers.Clear();

            // Проверим ввода папки
            if (!Directory.Exists(tbxBinPath.Text))
            {
                MessageBox.Show("Указана несуществующая папка.", "Ошибка!", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }
                

            var files = Directory.GetFiles(tbxBinPath.Text, "*.bin");
            
            foreach (var path in files)
            {
                // Читаем версию бинарника
                var binVer = BinaryCache<TradeInfo>.ReadVersion(path);

                // Получаем тикер из имени файла, и если тикер новый, добавляем имя тикера в общий список
                // Проверяем версии бинарников, если в двух версиях, то указываем обе версии.
                var name = Path.GetFileNameWithoutExtension(path);
                var tickerLen = name.Length - ".MM.dd.yyyy".Length;
                var ticker = name.Substring(0, tickerLen);      // В конце имени файла всегда ДАТА. Все что до даты, есть имя тикера для текущего брокера
                
                // Ищем в списке тикеров только что прочитанные, если он уже есть, 
                // и версия бинарника уже такая есть, то переходим к след файлу.
                var et = tickerInfos.Find(info => info.Ticker == ticker);
                if (et != null && et.Versions.Contains(binVer))
                    continue;

                // Если тикер уже есть но версия файла прочитана новая.
                if (et != null)
                    et.Versions.Add(binVer);

                // Если тикера еще нет, добавим новый тикер в список.
                if (et == null)
                    tickerInfos.Add(new TickerInfo(ticker, new List<int>(){binVer}));
            }

            tickerInfos.Sort((info1, info2) => string.Compare(info1.Ticker, info2.Ticker, StringComparison.InvariantCultureIgnoreCase));
            tickerInfos.ForEach(_tickers.Add);
        }

        private void btnSaveBin2Txt_Click(object sender, RoutedEventArgs e)
        {
            // Получим выбранный тикер, если ничего не выбрали то выходим.
            var tickerInfo = cbxTickers.SelectedItem as TickerInfo;
            if (tickerInfo == null)
                return;

            // Выключим кнопку записи в файл чтобы сто раз не жмакали
            ((Button) sender).IsEnabled = false;

            var sortedDictionary = new SortedDictionary<DateTime, string>();
            var binPath = tbxBinPath.Text;

            // Читаем все имена файлов содержащие выбранный тикер
            var files = Directory.GetFiles(binPath, tickerInfo.Ticker + "*.bin").Select(Path.GetFileName).ToArray();

            // Получаем список соответствий. Дата - файл и сортируем это дело по дате.
            foreach (var name in files)
            {
                var parts = name.Split(new char[] {'.'}, StringSplitOptions.RemoveEmptyEntries);
                var partsNum = parts.Count();
                var dateStr = "{0}.{1}.{2}".Put(parts[partsNum - 1 - 2], parts[partsNum - 1 - 3], parts[partsNum - 1 - 1]);
                var date = DateTime.ParseExact(dateStr, "dd.MM.yyyy", CultureInfo.CurrentCulture );

                sortedDictionary.Add(date, name);
            }
            

            // Инициализируем прогресс бар
            prgbBin2Txt.Minimum = 0;
            prgbBin2Txt.Maximum = sortedDictionary.Count - 1;
            prgbBin2Txt.Value = 0;
            prgbBin2Txt.IsIndeterminate = false;


            // теперь берем даты поочереди и переводим в текст
            var task = Task.Factory.StartNew(() =>
                {
                    var cache = new BinaryCache<TradeInfo>();
                    var streamW = new StreamWriter(@"{0}\{1}.{2}".Put(binPath, tickerInfo.Ticker, "trd"), false);

                    var txtTemplate = "{0:dd.MM.yyyy HH:mm:ss.fff};{1};{2};{3};{4};{5};{6};{7}";
                    streamW.WriteLine(txtTemplate.Put("Date", "TradeNo", "Price", "Volume", "Direction", "Ask", "Bid", "Interest"));

                    // Для каждого бинарного файла тикер, производим процедуру записи в ТХТ файл.
                    foreach (var item in sortedDictionary)
                    {
                        var path = @"{0}\{1}".Put(binPath, item.Value);
                        var version = BinaryCache<TradeInfo>.ReadVersion(path);
                        if (version == 0)
                            throw new Exception("Не удалось получить версию формата файла.");

                        var trades = cache.LoadCached(path, version);

                        var ic = CultureInfo.InvariantCulture;
                        trades.ForEach(
                            t =>
                            streamW.WriteLine(txtTemplate.Put(t.Date, t.TradeNo, t.Price.ToString(ic), t.Quantity,
                                                              t.Direction, t.Ask.ToString(ic), t.Bid.ToString(ic),
                                                              t.OpenInterest)));

                        // Обновляем прогресс бар
                        this.Dispatcher.BeginInvoke(new Action(() => prgbBin2Txt.Value += 1), DispatcherPriority.Normal);
                    }

                    // Скидываем на диск и закрываем потоки.
                    streamW.Flush();
                    streamW.Close();

                    // Включим кнопку записи в файл
                    this.Dispatcher.BeginInvoke(new Action(() => ((Button) sender).IsEnabled = true), DispatcherPriority.Normal);
                }).ContinueWith(task1 =>
                    {
                        if (task1.Exception != null) 
                            Trace.Fail(task1.Exception.ToString());
                    }, TaskContinuationOptions.OnlyOnFaulted);
        }

        private void btnBrowseTxt_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new OpenFileDialog() {Multiselect = false};
            if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                tbxTxtPath.Text = dialog.FileName;
            }
        }

        private void btnSaveTxt2Bin_Click(object sender, RoutedEventArgs e)
        {
            // Выключим кнопку записи в файл чтобы сто раз не жмакали
            ((Button) sender).IsEnabled = false;

            // Читаем ввод пользователя
            var binVersion = int.Parse(tbxBinVer.Text);
            var binPrefix = tbxBinPrefix.Text;
            var filePath = tbxTxtPath.Text;

            // Проверим ввода файла
            if (!File.Exists(filePath))
            {
                MessageBox.Show("Указан несуществующий файл.", "Ошибка!", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }
            
            var dirPath = Path.GetDirectoryName(filePath);

            // Инициализируем прогресс бар
            prgbTxt2Bin.IsIndeterminate = true;

            // Запустим процесс записи данных в отдельном потоке.
            var task = Task.Factory.StartNew(() =>
                {
                    var cache = new BinaryCache<TradeInfo>();
                    var sr = new StreamReader(filePath);

                    var trades = new List<TradeInfo>();
                    var currDate = DateTime.MinValue;

                    // Пропустим первую строчку ибо в ней заголовки
                    sr.ReadLine();


                    while (!sr.EndOfStream)
                    {
                        var line = sr.ReadLine();
                        var parts = line.Split(new char[] { ';' }, StringSplitOptions.RemoveEmptyEntries);

                        // Date;TradeNo;Price;Volume;Direction;Ask;Bid;Interest
                        var trade = new TradeInfo();
                        var ic = CultureInfo.InvariantCulture;
                        trade.Date = DateTime.ParseExact(parts[0], "dd.MM.yyyy HH:mm:ss.fff", ic);
                        trade.TradeNo = long.Parse(parts[1], ic);
                        trade.Price = float.Parse(parts[2], ic);
                        trade.Quantity = float.Parse(parts[3], ic);
                        TradeDirection direction;
                        var ok = Enum.TryParse(parts[4], true, out direction);
                        trade.Direction = ok ? direction : TradeDirection.Unknown;
                        trade.Ask = float.Parse(parts[5], ic);
                        trade.Bid = float.Parse(parts[6], ic);
                        trade.OpenInterest = float.Parse(parts[7], ic);

                        trade.StepPrice = 1F;

                        if (currDate == DateTime.MinValue)
                            currDate = trade.Date.Date;

                        // Если взяли сделку со следующего дня, пишем то что есть в файл.
                        if (trade.Date.Date > currDate)
                        {
                            var path = Path.Combine(dirPath, @"{0}.{1:MM.dd.yyyy}.bin".Put(binPrefix, currDate));
                            cache.SaveCached(path, trades, binVersion);

                            trades.Clear();
                            currDate = trade.Date.Date;

                            // Обновляем прогресс бар
                            //this.Dispatcher.BeginInvoke(new Action(() => prgbTxt2Bin.Value += 1), DispatcherPriority.Normal);
                        }

                        trades.Add(trade);
                    }

                    // После завершения работы пишем в файл те сделки которые еще не записаны
                    if (trades.Count > 0)
                    {
                        var path = Path.Combine(dirPath, @"{0}.{1:MM.dd.yyyy}.bin".Put(binPrefix, currDate));
                        cache.SaveCached(path, trades, binVersion);

                        trades.Clear();
                    }

                    sr.Close();

                    // Обновляем прогресс бар
                    this.Dispatcher.BeginInvoke(new Action(() =>
                        {
                            prgbTxt2Bin.IsIndeterminate = false;
                            prgbTxt2Bin.Maximum = 1000;
                            prgbTxt2Bin.Value = 1000;
                        }), DispatcherPriority.Normal);

                    // Включим кнопку записи в файл
                    this.Dispatcher.BeginInvoke(new Action(() => ((Button) sender).IsEnabled = true), DispatcherPriority.Normal);
                }).ContinueWith(task1 =>
                    {
                        if (task1.Exception != null) 
                            Trace.Fail(task1.Exception.ToString());
                    }, TaskContinuationOptions.OnlyOnFaulted); ;
        }
    }
}
